/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState } from 'react';
import { AccountCircle } from '@mui/icons-material';
import logo_white from './gp-logo-white.svg';
import logo from './gp-logo.svg';

import { updateCurrentDatabase } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import { useAppDispatch, useAuthorizationCache, useDatabaseAPI, useSessionCache } from '@graphpolaris/shared/lib/data-access';
import { DatabaseMenu } from './databasemenu';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { SearchBar } from './search/SearchBar';
import DatabaseSelector from './DatabaseManagement/DatabaseSelector';

export interface NavbarComponentProps {
  // changeColourPalette: () => void; FIXME move to redux
}

/** NavbarComponentState is an interface containing the type of visualizations. */
export interface NavbarComponentState {
  // The anchor for rendering the user data (userID, sessionID, add database, etc) menu
  userMenuAnchor?: Element;
  // The anchor for rendering the menu for selecting a database to use
  selectDatabaseMenuAnchor?: Element;
  deleteDatabaseMenuAnchor?: Element;
  // Determines if the addDatabaseForm will be shown
  showAddDatabaseForm: boolean;
}

export interface NavbarSubComponentState {
  changeDb?: String;
  deleteDb?: String;
}

/** NavbarComponent is the View implementation for Navbar */
export const Navbar = (props: NavbarComponentProps) => {
  const auth = useAuthorizationCache();
  const session = useSessionCache();
  const api = useDatabaseAPI();
  const dispatch = useAppDispatch();
  const [menuOpen, setMenuOpen] = useState(false);
  const [subMenuOpen, setSubMenuOpen] = useState<string | undefined>(undefined);

  const currentLogo = !'dark' ? logo_white : logo; // TODO: support dark mode

  const buildInfo = import.meta.env.GRAPHPOLARIS_VERSION;

  return (
    <div className="w-full h-auto px-5">
      <div title="GraphPolaris" className="navbar flex items-center justify-between w-auto gap-2 xl:gap-10">
        <a href="https://graphpolaris.com/" target="_blank" className="xl:w-fit w-[5rem] shrink-0">
          <img src={currentLogo} alt="GraphPolaris" className="h-9" />
        </a>

        <DatabaseSelector />

        <div>
          <SearchBar />

          <div className="w-fit">
            <div className="menu-walkthrough">
              <button
                tabIndex={0}
                className="btn btn-circle btn-ghost hover:bg-gray-200"
                onClick={(event) => {
                  setMenuOpen(!menuOpen);
                  setSubMenuOpen(undefined);
                }}
              >
                <AccountCircle htmlColor="black" />
              </button>
              {menuOpen && (
                <>
                  <div
                    className="z-10 bg-transparent absolute w-screen h-screen top-0 left-0"
                    onClick={() => {
                      setMenuOpen(false);
                      setSubMenuOpen(undefined);
                    }}
                  ></div>
                  <ul
                    tabIndex={0}
                    className="z-20 dropdown-content menu absolute right-4 p-2 shadow-xl bg-offwhite-100 rounded-box max-w-52 w-52"
                  >
                    {auth.authorized ? (
                      <div className="w-full">
                        <div className="menu-title">
                          <h2>user: {auth.username}</h2>
                          <h3 className="text-xs break-words">session: {auth.sessionID}</h3>
                        </div>

                        <div className="menu-title">
                          <div className="absolute left-0 h-0.5 w-full bg-offwhite-300"></div>
                          <h3 className="text-xs mt-3">Version: {buildInfo}</h3>
                        </div>
                      </div>
                    ) : (
                      <div className="w-full">
                        <div className="menu-title">
                          <h2>user: {auth.username}</h2>
                          <h3 className="text-xs">session: {auth.sessionID}</h3>
                        </div>
                        <div>
                          <button
                            className="btn btn-ghost"
                            onClick={() => {
                              setMenuOpen(false);
                            }}
                          >
                            <span>Login</span>
                            {/* !TODO */}
                          </button>
                        </div>
                        <div className="menu-title">
                          <div className="absolute left-0 h-0.5 w-full bg-offwhite-300"></div>
                          <h3 className="text-xs mt-3">Version: {buildInfo}</h3>
                        </div>
                      </div>
                    )}
                  </ul>
                </>
              )}
            </div>
          </div>
        </div>
      </div>
    </div>
  );
};
